function results = SDPD_CF_Wvt_DS(ya,yb,xa,xb,Wa,Wb,shift,si)

% a = after the shift, b = before the shift
% Obtain basic dimension variables used throughout the function
na = size(Wa,1);
nb = size(Wb,1);
nta = size(ya,1);
ntb = size(yb,1);
kxa = size(xa,2);
kxb = size(xb,2);
ta = nta/na-1;
tb = ntb/nb-1;
nta = na*ta;
ntb = nb*tb;
t = ta + tb;
nt = nta + ntb;
 
% Obtain time lag, spatial lag and time spatial lag for the before and
% after subsets
yta = ya(na+1:na+nta);
ytla = ya(1:nta);
ysla = zeros(nta,1);
ystla = zeros(nta,1);
for i = 1:ta
    ysla(1+(i-1)*na:i*na) = Wa(:,1+i*na:(i+1)*na)*yta(1+(i-1)*na:i*na);
    ystla(1+(i-1)*na:i*na) = Wa(:,1+(i-1)*na:i*na)*ytla(1+(i-1)*na:i*na);
end

ytb = yb(nb+1:nb+ntb);
ytlb = yb(1:ntb);
yslb = zeros(ntb,1);
ystlb = zeros(ntb,1);
for i = 1:tb
    yslb(1+(i-1)*nb:i*nb) = Wb(:,1+i*nb:(i+1)*nb)*ytb(1+(i-1)*nb:i*nb);
    ystlb(1+(i-1)*nb:i*nb) = Wb(:,1+(i-1)*nb:i*nb)*ytlb(1+(i-1)*nb:i*nb);
end

% Use function to subtract the region mean of the values. Importantly, the 
% for all provinces except Flevoland all observations are used, for 
% Flevoland only the observations after the shift are used
results = DS_demean(yta,ytb,na,nb,ta,tb,nta,ntb,shift,si);
yta = results.ya;
ytb = results.yb;

results = DS_demean(ytla,ytlb,na,nb,ta,tb,nta,ntb,shift,si);
ytla = results.ya;
ytlb = results.yb;

results = DS_demean(ysla,yslb,na,nb,ta,tb,nta,ntb,shift,si);
ysla = results.ya;
yslb = results.yb;

results = DS_demean(ystla,ystlb,na,nb,ta,tb,nta,ntb,shift,si);
ystla = results.ya;
ystlb = results.yb;

% The x variable also has to be demeaned, here are the steps
% reformat to make demeaning per region easier
xa = reshape(xa,na,ta,kxa);
xb = reshape(xb,nb,tb,kxb);

% Seperate the x matrices into Flevoland and other provinces
xOP = cat(2, xb, xa([1:si-1, si+1:end], :,[1:si-1, si+1:na+si-1, na+si+1:end]));
xFl = xa(:,:,[si, si+na]);

% Demean the observations
for i = 1:kxb
    temp = mean(xOP(:,:,i)')';
    temp = temp*ones(1,t);
    xOP(:,:,i) = xOP(:,:,i)-temp;
end

for i = 1:size(xFl,3)
    temp = mean(xFl(:,:,i)')';
    temp = temp*ones(1,size(xFl,2));
    xFl(:,:,i) = xFl(:,:,i)-temp;
end

% reassemble the before and after matrices, we are working with a 3-d array
% which leads us to take an intermediary step
temp = zeros(1,t,kxb);
temp = cat(1, xOP(1:si-1,:,:), temp, xOP(si:end,:,:));
xa = cat(3, temp(:,shift:end,1:si-1), xFl(:,:,1), ...
    temp(:,shift:end,si:nb+si-1), xFl(:,:,2), temp(:,shift:end,nb+si:end));
xb = xOP(:,1:shift-1,:);

% Reformat into a matrix
xta = reshape(xa,nta, kxa);
xtb = reshape(xb,ntb,kxb);
    
% Obtain zt variables
zta = [ytla ystla xta];
kza = size(zta,2);

ztb = [ytlb ystlb xtb];
kzb = size(ztb,2);

% Obtain rho estimate van concentrated log likelihood
options = optimset('fminsearch');
parm = 0;
% We concentrate the likelihood here by substituting e'*e for sige, e'*e is
% calculated by obtaining b by substituting through rho. This way the
% function only depends on rho and we can estimate in a simple manner
[pout,like,exitflag,output]=fminsearch('lee_f_sar_sdpd_Wt_DS',parm,options,yta,ytb,ysla,yslb,zta,ztb,Wa,Wb);
rho = pout(1,1);
rho = 2/(1+exp(-rho))-1;

% Obtain the other estimates based on the found rho, by substituting the
% estimate back
% Since beta consists 2 more coefficients the si needs to be shifted by 2 
sit = si + 2;
Syta = yta-rho*ysla;
betaa = zta\Syta;
Sytb = ytb-rho*yslb;
betab = ztb\Sytb;

% We need one estimate of beta which contains all 12 provinces
% What we do is take the weighted mean of the two beta estimates, leaving
% the Flevoland estimate totally determined by the after shift observations.
temp = zeros(kza,1);
temp(1:sit-1) = betab(1:sit-1);
temp(sit) = betaa(sit);
temp(sit+1:sit+nb)= betab(sit:sit+nb-1);
temp(sit+na)= betaa(sit+na);
temp(sit+na+1:end)= betab(sit+nb:end);
betab = temp;
% We take the weighted average of the before and after beta estimate
beta = nta/(nta+ntb)*betaa + ntb/(nta+ntb)*betab;

% Derive the residuals
ea = Syta- zta*beta;
eb = Sytb- ztb*beta([1:sit-1 sit+1:sit+nb sit+na+1:end]);
% Combine the residuals to obtain one estimate for the sige
e = [eb' ea']';
sigu = e'*e;
sige = sigu/nt;
theta = [beta;rho;sige];

% Obtain the covariance matrix
xpx = zeros(kza+2,kza+2);

% Calculate the traces for these time varying weight matrices, we can add
% the before and after trace
trG=0;
trG2=0;
for i=1:ta
    Sta=speye(na)-rho*Wa(:,1+i*na:(i+1)*na);
    Wta=Wa(:,1+i*na:(i+1)*na);
    Gta=Wta*inv(Sta);
    trG=trG+trace(Gta);
    trG2=trG2+trace(Gta^2);
end
for i=1:tb
    Stb=speye(nb)-rho*Wb(:,1+i*nb:(i+1)*nb);
    Wtb=Wb(:,1+i*nb:(i+1)*nb);
    Gtb=Wtb*inv(Stb);
    trG=trG+trace(Gtb);
    trG2=trG2+trace(Gtb^2);
end

% For the covariance matrix we need to calculate some inner products, to be
% able to add the innerproducts of both the before and after matrix we need
% to enlarge the before inner product, we fill in zeros for the columns
% corresponding to Flevoland (column 7 and 19)
% zt zt innerproduct
zta2 = zta'*zta;
ztb2 = ztb'*ztb;
temp = zeros(kza);
temp(1:sit-1,1:sit-1) = ztb2(1:sit-1,1:sit-1);
temp(sit+1:sit+nb,1:sit-1) = ztb2(sit:sit+nb-1,1:sit-1);
temp(1:sit-1,sit+1:sit+nb) = ztb2(1:sit-1,sit:sit+nb-1);
temp(sit+1:sit+nb,sit+1:sit+nb) = ztb2(sit:sit+nb-1,sit:sit+nb-1);
temp(sit+nb+2:kza,1:sit-1) = ztb2(sit+nb:nb*2+2,1:sit-1);
temp(sit+nb+2:kza,sit+1:sit+nb) = ztb2(sit+nb:nb*2+2,sit:sit+nb-1);
temp(1:sit-1,sit+nb+2:kza) = ztb2(1:sit-1,sit+nb:nb*2+2);
temp(sit+1:sit+nb,sit+nb+2:kza) = ztb2(sit:sit+nb-1,sit+nb:nb*2+2);
temp(sit+nb+2:kza,sit+nb+2:kza) = ztb2(sit+nb:nb*2+2,sit+nb:nb*2+2);
ztb2 = temp;

% zt ysl innerproduct
zta_ysla = zta'*ysla;
ztb_yslb = ztb'*yslb;
temp = zeros(kza,1);
temp(1:sit-1) = ztb_yslb(1:sit-1);
temp(sit+1:sit+nb) = ztb_yslb(sit:sit+nb-1);
temp(sit+nb+2:end) = ztb_yslb(sit+nb:end);
ztb_yslb = temp;

% The terms of the covariance matrix
% Formulas are taken from Lee and Yu 2012
% note that the expectation still need to be taken in this formulation
term1 = trG2;
term2 = (1/sige)*(ysla'*ysla + yslb'*yslb);
xpx(1:kza,1:kza) = (1/sige)*(zta2 + ztb2);
xpx(kza+1,kza+1) = term1+term2;
xpx(kza+2,kza+2) = nt/(2*sige*sige);
xpx(1:kza,kza+1) = (1/sige)*(zta_ysla + ztb_yslb);
xpx(kza+1,1:kza) = xpx(1:kza,kza+1)';
xpx(kza+1,kza+2) = (1/sige)*trG;
xpx(kza+2,kza+1) = xpx(kza+1,kza+2)';

xpx=xpx/nt;
xpxi = invpd(xpx);

% The diagonal is used for the std and tstat
tmp = diag(abs(xpxi));

% Obtain standard deviations and t-statistics
std = sqrt(tmp)/sqrt(nt);
tstat = sqrt(nt)*theta./sqrt(tmp(1:kza+2,1));

% We need to correct for bias. To do so we calculate the values of eq (14) 
% from Lee and Yu (2012)
% First we calculate S, G, A which will be used later
results = DS_bias_prep(kza,na,ta,Wa,rho,theta);
Sa = results.S;
Ga = results.G; 
Aa = results.A;

results = DS_bias_prep(kzb,nb,tb,Wb,rho,theta);
Sb = results.S;
Gb = results.G; 
Ab = results.A;

% Now we calculate the bias terms
results = DS_bias_calc(na,ta,Wa,Ga,Sa,Aa,theta);
bias1a = results.bias1;
bias2a = results.bias2;
bias3a = results.bias3;

results = DS_bias_calc(nb,tb,Wb,Gb,Sb,Ab,theta);
bias1b = results.bias1;
bias2b = results.bias2;
bias3b = results.bias3;

% See matrix (14) in Lee and Yu 2012
bias(1,1) = (1/nta)*bias1a + (1/ntb)*bias1b;
bias(2,1) = (1/nta)*bias2a + (1/ntb)*bias2b;
bias(kza+1,1) = (1/nta)*bias3a + (1/ntb)*bias3b +(1/nt)*trG;
bias(kza+2,1) = inv(2*sige);

% The bias corrected estimates
theta1 = theta+xpxi*bias/(ta+tb);

% Now obtain new covariance matrix based on the bias corrected estimates
% This is just line 81-144 done again for the new theta1 estimates, note
% that we do not need to calculate the cross product terms again
rho1 = theta1(kza+1);
beta1 = theta1(1:kza);
sige1 = theta1(kza+2);

xpx = zeros(kza+2,kza+2);
trG=0;
trG2=0;
for i=1:ta
    Sta=speye(na)-rho1*Wa(:,1+i*na:(i+1)*na);
    Wta=Wa(:,1+i*na:(i+1)*na);
    Gta=Wta*inv(Sta);
    trG=trG+trace(Gta);
    trG2=trG2+trace(Gta^2);
end
for i=1:tb
    Stb=speye(nb)-rho1*Wb(:,1+i*nb:(i+1)*nb);
    Wtb=Wb(:,1+i*nb:(i+1)*nb);
    Gtb=Wtb*inv(Stb);
    trG=trG+trace(Gtb);
    trG2=trG2+trace(Gtb^2);
end

term1 = trG2;
term2 = (1/sige1)*(ysla'*ysla + yslb'*yslb);
xpx(1:kza,1:kza) = (1/sige1)*(zta2 + ztb2);
xpx(kza+1,kza+1) = term1+term2;
xpx(kza+2,kza+2) = nt/(2*sige1*sige1);
xpx(1:kza,kza+1) = (1/sige1)*(zta_ysla + ztb_yslb);
xpx(kza+1,1:kza) = xpx(1:kza,kza+1)';
xpx(kza+1,kza+2) = (1/sige1)*trG;
xpx(kza+2,kza+1) = xpx(kza+1,kza+2)';

xpx = xpx/nt;
xpxi = invpd(xpx);
tmp = diag(abs(xpxi));

SIGi1 = xpxi;
SIG1 = invpd(SIGi1);

% Obtain the new std and tstat also calculate the residuals
std1 = sqrt(tmp)/sqrt(nt);
tstat1 = sqrt(nt)*theta1./sqrt(tmp(1:kza+2,1));
respaul1a = yta-theta1(kza+1,1)*ysla-zta*theta1(1:kza,1);
respaul1b = ytb-theta1(kza+1,1)*yslb-ztb*theta1([1:sit-1, sit+1:na+sit-1 na+sit+1:kza]);
respaul1 = [respaul1b' respaul1a']';

% Obtain log likelihood
e = respaul1;
ldet=0;
for i=1:ta
    St=speye(na)-rho*Wa(:,1+i*na:(i+1)*na);
    dSt=log(det(St));
    ldet=ldet+dSt;
end
for i=1:tb
    St=speye(nb)-rho*Wb(:,1+i*nb:(i+1)*nb);
    dSt=log(det(St));
    ldet=ldet+dSt;
end

llike1 = -(nt/2)*log(2*pi) - (nt/2)*log(sige1) + ldet - (1/(2*sige1))*e'*e;

% Obtain the R squared
rsqr1 = e'*e;
yt = cat(1,yta,ytb);
rsqr2 = (yt-mean(yt))'*(yt-mean(yt));
rsqr = 1-rsqr1/rsqr2;

% Collect results
results.iter = output.iterations;
results.beta = beta;
results.rho = rho;
results.resid = e;
results.yhat = [yta' ytb']'-e;
results.sige = sige;
results.theta = theta;
results.tstat = tstat;
results.std = std;
results.SIG1 = SIG1;
results.SIGi1 = SIGi1;
results.beta1 = beta1;
results.rho1 = rho1;
results.sige1 = sige1;
results.theta1 = theta1;
results.std1 = std1;
results.tstat1 = tstat1;
results.respaul1 = respaul1;
results.llike1 = llike1;
results.rsqr = rsqr;
results.varcov = SIGi1/nt;