function results = SDPD_CF_Wvt_base(y,x,W)

% Obtain basic dimension variables used throughout the function
n = size(W,1);
nt = size(y,1);
kx = size(x,2);
t = nt/n-1;
nt = n*t;

% Obtain time lag, spatial lag and time spatial lag
yt = y(n+1:n+nt);
ytl = y(1:nt);
ysl = zeros(nt,1);ystl = zeros(nt,1);
for i = 1:t
    ysl(1+(i-1)*n:i*n) = W(:,1+i*n:(i+1)*n)*yt(1+(i-1)*n:i*n);
    ystl(1+(i-1)*n:i*n) = W(:,1+(i-1)*n:i*n)*ytl(1+(i-1)*n:i*n);
end

% Demean all variables to concentrate out the region fixed effects 
yt = reshape(yt,n,t);
temp = mean(yt')';
temp = temp*ones(1,t);
yt = yt-temp;
yt = reshape(yt,nt,1);

ytl = reshape(ytl,n,t);
temp = mean(ytl')';
temp = temp*ones(1,t);
ytl = ytl-temp;
ytl = reshape(ytl,nt,1);

ysl = reshape(ysl,n,t);
temp = mean(ysl')';
temp = temp*ones(1,t);
ysl = ysl-temp;
ysl = reshape(ysl,nt,1);

ystl = reshape(ystl,n,t);
temp = mean(ystl')';
temp = temp*ones(1,t);
ystl = ystl-temp;
ystl = reshape(ystl,nt,1);

x = reshape(x,n,t,kx);
for i = 1:kx
    temp = mean(x(:,:,i)')';
    temp = temp*ones(1,t);
    x(:,:,i) = x(:,:,i)-temp;
end
xt = reshape(x,nt,kx);

% Prepare z vector for later
zt = [ytl ystl xt];
kz = size(zt,2);

% Obtain rho estimate van concentrated log likelihood
options = optimset('fminsearch');
parm = 0;
% We concentrate the likelihood here by substituting e'*e for sige, e'*e is
% calculated by obtaining b by substituting through rho. This way the
% function only depends on rho and we can estimate in a simple manner
[pout,like,exitflag,output]=fminsearch('lee_f_sar_sdpd_Wt',parm,options,yt,ysl,zt,W);
rho = pout(1,1);
rho = 2/(1+exp(-rho))-1;

% Obtain the other estimates based on the found rho, by substituting the
% estimate back
Syt = yt-rho*ysl;
beta = zt\Syt;
e = Syt- zt*beta;
sigu = e'*e;
sige = sigu/nt;
theta = [beta;rho;sige];


% Obtain the covariance matrix
xpx = zeros(kz+2,kz+2);

% Calculate the traces for these time varying weight matrices
trG=0;
trG2=0;
for i=1:t
    St=speye(n)-rho*W(:,1+i*n:(i+1)*n);
    Wt=W(:,1+i*n:(i+1)*n);
    Gt=Wt*inv(St);
    trG=trG+trace(Gt);
    trG2=trG2+trace(Gt^2);
end

% The terms of the covariance matrix
% Formulas are taken from Lee and Yu 2012
term1 = trG2;
term2 = (1/sige)*ysl'*ysl;
xpx(1:kz,1:kz) = (1/sige)*(zt'*zt);
xpx(kz+1,kz+1) = term1+term2;
xpx(kz+2,kz+2) = nt/(2*sige*sige);
xpx(1:kz,kz+1) = (1/sige)*(zt'*ysl);
xpx(kz+1,1:kz) = xpx(1:kz,kz+1)';
xpx(kz+1,kz+2) = (1/sige)*trG;
xpx(kz+2,kz+1) = xpx(kz+1,kz+2)';

xpx=xpx/nt;
xpxi = invpd(xpx);

% The diagonal is used for the std and tstat
tmp = diag(abs(xpxi));

% Obtain standard deviations and t-statistics
std = sqrt(tmp)/sqrt(n*t);
tstat = sqrt(n*t)*theta./sqrt(tmp(1:kz+2,1));

% We need to correct for bias. To do so we calculate the values of eq (14) 
% from Lee and Yu (2012)
% First we calculate S, G, A which will be used later
In = eye(n);
bias = zeros(kz+2,1);
S = kron(ones(1,t+1),In)-rho*W;
G = zeros(n,n*t);
for s=1:t
    G(:,1+(s-1)*n:s*n)=W(:,1+s*n:(s+1)*n)*inv(S(:,1+s*n:(s+1)*n));
end

A = zeros(n,n*t); 
gamma_coff = beta(1); 
rho_coff = beta(2);
for s=1:t
    A(:,1+(s-1)*n:s*n) = inv(S(:,1+s*n:(s+1)*n))*(gamma_coff*In+rho_coff*W(:,1+(s-1)*n:s*n));
end

% Now we calculate the bias terms
bias1 = 0;
bias2 = 0;
bias3 = 0;

for s = 1:t-1
    temp1a = eye(n); 
    temp2a = W(:,1+s*n:(s+1)*n);
    temp3a = G(:,1+s*n:(s+1)*n)*(gamma_coff*temp1a+rho_coff*temp2a);
    for h = 2:t-s
        temp1b = eye(n);
        for j = 1:h-1
            temp1b = temp1b*A(:,1+(s+j-1)*n:(s+j)*n);
        end
        temp1a = temp1a+temp1b;
        temp2a = temp2a+W(:,1+(s+h-1)*n:(s+h)*n)*temp1b;
        temp3a = temp3a+gamma_coff*G(:,1+(s+h-1)*n:(s+h)*n)*temp1b+rho_coff*G(:,1+(s+h-1)*n:(s+h)*n)*W(:,1+(s+h-1)*n:(s+h)*n)*temp1b;
    end
    % See matrix (14) in Lee and Yu 2012, Jn not needed because no time
    % fixed effects
    bias1 = bias1+trace(inv(S(:,1+s*n:(s+1)*n))*temp1a);
    bias2 = bias2+trace(inv(S(:,1+s*n:(s+1)*n))*temp2a);
    bias3 = bias3+trace(inv(S(:,1+s*n:(s+1)*n))*temp3a);
end
    
% See matrix (14) in Lee and Yu 2012
bias(1,1) = (1/nt)*bias1;
bias(2,1) = (1/nt)*bias2;
bias(kz+1,1) = (1/nt)*bias3+(1/nt)*trG;
bias(kz+2,1) = inv(2*sige);

% The bias corrected estimates
theta1 = theta+xpxi*bias/t;

% Now obtain new covariance matrix based on the bias corrected estimates
% This is just line 77-103 done again for the new theta1 estimates
rho1 = theta1(kz+1);
beta1 = theta1(1:kz);
sige1 = theta1(kz+2);

xpx = zeros(kz+2,kz+2);
trG = 0;
trG2 = 0;
for i = 1:t
    St = speye(n)-rho1*W(:,1+i*n:(i+1)*n);
    Wt = W(:,1+i*n:(i+1)*n);
    Gt = Wt*inv(St);
    trG = trG+trace(Gt);
    trG2 = trG2+trace(Gt^2);
end

term1 = trG2;
term2 = (1/sige1)*ysl'*ysl;
xpx(1:kz,1:kz) = (1/sige1)*(zt'*zt);
xpx(kz+1,kz+1) = term1+term2;
xpx(kz+2,kz+2) = nt/(2*sige1*sige1);
xpx(1:kz,kz+1) = (1/sige1)*(zt'*ysl);
xpx(kz+1,1:kz) = xpx(1:kz,kz+1)';
xpx(kz+1,kz+2) = (1/sige1)*trG;
xpx(kz+2,kz+1) = xpx(kz+1,kz+2)';

xpx = xpx/nt;
xpxi = invpd(xpx);
tmp = diag(abs(xpxi));

SIGi1 = xpxi;
SIG1 = invpd(SIGi1);

% Obtain the new std and tstat also calculate the residuals
std1 = sqrt(tmp)/sqrt(n*t);
tstat1 = sqrt(n*t)*theta1./sqrt(tmp(1:kz+2,1));
respaul1 = yt-theta1(kz+1,1)*ysl-zt*theta1(1:kz,1);

% Obtain log likelihood
e = respaul1;
ldet=0;
for i=1:t
    St=speye(n)-rho1*W(:,1+i*n:(i+1)*n);
    dSt=log(det(St));
    ldet=ldet+dSt;
end
llike1 = -(n*t/2)*log(2*pi) - (n*t/2)*log(sige1) + ldet - (1/(2*sige1))*e'*e;

% Obtain the R squared
rsqr1 = e'*e;
rsqr2 = (yt-mean(yt))'*(yt-mean(yt));
rsqr = 1-rsqr1/rsqr2;

% Collect results
results.iter = output.iterations;
results.beta = beta;
results.rho = rho;
results.resid = e;
results.yhat = yt-e;
results.sige = sige;
results.theta = theta;
results.tstat = tstat;
results.std = std;
results.y = y;
results.SIG1 = SIG1;
results.SIGi1 = SIGi1;
results.beta1 = beta1;
results.rho1 = rho1;
results.sige1 = sige1;
results.theta1 = theta1;
results.std1 = std1;
results.tstat1 = tstat1;
results.respaul1 = respaul1;
results.llike1 = llike1;
results.rsqr = rsqr;
results.varcov = SIGi1/nt;