clear
% Load the data
A = xlsread('data.xlsx');
W1955_s=xlsread('Travel times.xlsx','1955');
W1970_s=xlsread('Travel times.xlsx','1970');
W1980_s=xlsread('Travel times.xlsx','1980');
W1990_s=xlsread('Travel times.xlsx','1990');
W2001_s=xlsread('Travel times.xlsx','2001');
W2012_s=xlsread('Travel times.xlsx','2012');

% a = after the shift, b = before the shift
% Obtain dimensions
N = 12;
Na = 12;
Nb = 11;
T = size(A,1)/Na;
nobs = Na*T;
% Shift represents the year after which the shift will take place
shift = 23;
% Si = shift_index, which is the index of the region that the shift
% happens around
si = 5;
Ta = T - shift;
Tb = shift;
nobsa = Na*Ta;
nobsb = Nb*Tb;

% Set up the for loop for best exponential decay parameter search
range = (1:10)/100;
k = length(range);
par_sum = zeros(k,1);
par_results = zeros(28,3,k);
par_LL = zeros(k,1);
par_R2 = zeros(k,1);
par_varcov = zeros(28,28,k);

for j=range % value distance decay 0.01 tot 0.10
% Exponential decay transformation, make diagional zero and row normalize
    W1955 = exp(-j*W1955_s); W1955 = W1955 - diag(diag(W1955)); W1955=normw(W1955); 
    W1970 = exp(-j*W1970_s); W1970 = W1970 - diag(diag(W1970)); W1970=normw(W1970); 
    W1980 = exp(-j*W1980_s); W1980 = W1980 - diag(diag(W1980)); W1980=normw(W1980); 
    W1990 = exp(-j*W1990_s); W1990 = W1990 - diag(diag(W1990)); W1990=normw(W1990); 
    W2001 = exp(-j*W2001_s); W2001 = W2001 - diag(diag(W2001)); W2001=normw(W2001); 
    W2012 = exp(-j*W2012_s); W2012 = W2012 - diag(diag(W2012)); W2012=normw(W2012);    
    
    % Prepare the before matrices for Flevoland by removing its row and
    % column
    W1995_b = W1955;
    W1995_b(si,:) = [];
    W1995_b(:,si) = [];
    W1995_b = normw(W1995_b); %Row normalize again, since we want the rows to still sum up to 1
    W1970_b = W1970;
    W1970_b(si,:) = [];
    W1970_b(:,si) = [];
    W1970_b = normw(W1970_b); %Row normalize again, since we want the rows to still sum up to 1
    
    % Reformat the spatial weight matrix to be time dependent
    W2a = zeros(Na, nobsa);
    W2b = zeros(Nb, nobsb);
    for i = 1:Tb
        if i < 20
            W2b(:,1+(i-1)*Nb:i*Nb) = W1995_b;
        else
            W2b(:,1+(i-1)*Nb:i*Nb) = W1970_b;
        end
    end
    for i = 1:Ta
        if i < 8
            W2a(:,1+(i-1)*Na:i*Na) = W1970;
        elseif i < 18
            W2a(:,1+(i-1)*Na:i*Na) = W1980;
        elseif i < 29
            W2a(:,1+(i-1)*Na:i*Na) = W1990;
        elseif i < 40
            W2a(:,1+(i-1)*Na:i*Na) = W2001;
        else
            W2a(:,1+(i-1)*Na:i*Na) = W2012;
        end
    end
    Wa = W2a;
    Wb = W2b;
    
    % Dependent variable
    y = A(:,2);
    % Split the data in a before and after subset
    ya = y(nobsb+shift+1:end);
    yb = y(1:nobsb+shift+1);
    temp = zeros(nobsb,1);
    % Remove the Flevoland observations
    for i = 1:Tb
        temp((i-1)*Nb+1:(i-1)*Nb+si-1) = yb((i-1)*Na+1:(i-1)*Na+si-1);
        temp((i-1)*Nb+si:i*Nb) = yb((i-1)*Na+si+1:i*Na);
    end
    yb = temp;
    
    % Obtaining the correct form for the x variables (national unemployment)
    x = zeros(nobs,N);
    for t = 1:T
        t1 = (t-1)*N+1;
        t2 = t*N;
        for i = 1:N
            x(t1-1+i,i) = A(t1-1+i,3);
        end  
    end
    % Split the data in a before and after subset
    xa = x(nobsb+shift+1:end,:);
    xb = x(1:nobsb+shift+1,:);
    temp = zeros(nobsb,Nb);
    % Remove the Flevoland observations
    for i = 1:Tb
        temp((i-1)*Nb+1:(i-1)*Nb+si-1,:) = xb((i-1)*Na+1:(i-1)*Na+si-1,[1:si-1, si+1:Na]);
        temp((i-1)*Nb+si:i*Nb,:) = xb((i-1)*Na+si+1:i*Na,[1:si-1, si+1:Na]);
    end
    xb = temp;
    % Current and time lagged values are used
    xa = [xa(Na+1:end,:) xa(1:end-Na,:)];
    xb = [xb(Nb+1:end,:) xb(1:end-Nb,:)];
    
    % Call the function running the estimation
    results = SDPD_CF_Wvt_DS(ya,yb,xa,xb,Wa,Wb,shift,si);
    % Stability check
    par_sum(round(j*100,0),1) = sum(results.theta1([1,2,size(results.theta1,1)-1]));
    % Coefficient results
    par_results(:,:,round(j*100,0)) = [results.theta1, results.std1, results.tstat1];
    % Log likelihood
    par_LL(round(j*100,0),1) = results.llike1;
    % R squared
    par_R2(round(j*100,0),1) = results.rsqr;
    % varcov 
    par_varcov(:,:,round(j*100,0)) = results.varcov;
end
% Subset to only keep the stable results
stable = par_sum<1; % yields 10
display('full model')
% Results for the best model
stability=round(par_sum(10),3)
round(par_results(:,:,10),3)
LogL=round(par_LL(10),3)
R2=round(par_R2(10),3)
varcov=par_varcov(:,:,10);
par=par_results(:,1,10);
% Obtain the gamma estimates based on the beta4 and beta5 estimates
gamma1 = gamma_Paul(par,varcov,N);
% Confidence intervals of gamma estimates
%[round(gamma1.b4b(:,1),3) round(gamma1.b4b(:,2),3)]
%[round(gamma1.b5b(:,1),3) round(gamma1.b5b(:,2),3)]
[round(gamma1.b45b(:,1),3) round(gamma1.b45b(:,2),3)]
%
% Before and after the structural break
% 
% For the pre break subset we have that the Flevoland shift is
% present so we estimate using the dimension-shifting procedure
% af represents the distance between the break and the shift
k_max=31;
af = k_max - shift;
yb_pre_k = yb;
xb_pre_k = xb;
Wb_pre_k = Wb;
ya_pre_k = ya(1:af*12);
xa_pre_k = xa(1:(af-1)*12,:);
Wa_pre_k = Wa(:,1:af*12);
results_pre_k = SDPD_CF_Wvt_DS(ya_pre_k,yb_pre_k,xa_pre_k,xb_pre_k,Wa_pre_k,Wb_pre_k,shift,si);
display('Period up to and including 1980')
stability = sum(results_pre_k.theta1([1,2,size(results_pre_k.theta1,1)-1]))
round([results_pre_k.theta1, results_pre_k.std1, results_pre_k.tstat1],3)
LogL = round(results_pre_k.llike1,3)
R2 = round(results_pre_k.rsqr,3)
varcov_pre= results_pre_k.varcov;
par_pre=results_pre_k.theta1;
% Confidence intervals of gamma estimates
gamma_pre = gamma_Paul(par_pre,varcov_pre,N);
%[round(gamma_pre.b4b(:,1),3) round(gamma_pre.b4b(:,2),3)]
%[round(gamma_pre.b5b(:,1),3) round(gamma_pre.b5b(:,2),3)]
[round(gamma_pre.b45b(:,1),3) round(gamma_pre.b45b(:,2),3)]

% Since the break is after the shift we have that no shift appears
% in the post-break subset, we can use the base estimation procedure
ya_post_k = ya(af*12+1:end);
xa_post_k = xa(af*12+1:end,:);
Wa_post_k = Wa(:,af*12+1:end);
results_post_k = SDPD_CF_Wvt_base(ya_post_k,xa_post_k,Wa_post_k);
display(['Period from 1981 onwards'])
stability = sum(results_post_k.theta1([1,2,size(results_post_k.theta1,1)-1]))
round([results_post_k.theta1,results_post_k.std1, results_post_k.tstat1],3)
LogL = round(results_post_k.llike1,3)
R2 = round(results_post_k.rsqr,3)
varcov_post= results_post_k.varcov;
par_post=results_post_k.theta1;
% Confidence intervals of gamma estimates
gamma_post = gamma_Paul(par_post,varcov_post,N);
%[round(gamma_post.b4b(:,1),3) round(gamma_post.b4b(:,2),3)]
%[round(gamma_post.b5b(:,1),3) round(gamma_post.b5b(:,2),3)]
[round(gamma_post.b45b(:,1),3) round(gamma_post.b45b(:,2),3)]

% Suppose af=13
af=13
ya_1986_k = ya(af*12+1:end);
xa_1986_k = xa(af*12+1:end,:);
Wa_1986_k = Wa(:,af*12+1:end);
results_1986_k = SDPD_CF_Wvt_base(ya_1986_k,xa_1986_k,Wa_1986_k);
display(['Period from 1986 onwards'])
stability = sum(results_1986_k.theta1([1,2,size(results_1986_k.theta1,1)-1]))
round([results_1986_k.theta1,results_1986_k.std1, results_1986_k.tstat1],3)
LogL = round(results_1986_k.llike1,3)
R2 = round(results_1986_k.rsqr,3)
varcov_1986= results_1986_k.varcov;
par_1986=results_1986_k.theta1;
% Confidence intervals of gamma1 estimates
gamma_1986 = gamma_Paul(par_1986,varcov_1986,N);
%[round(gamma_1986.b4b(:,1),3) round(gamma_1986.b4b(:,2),3)]
%[round(gamma_1986.b5b(:,1),3) round(gamma_1986.b5b(:,2),3)]
[round(gamma_1986.b45b(:,1),3) round(gamma_1986.b45b(:,2),3)]
%
% Comparison
%
display('Comparison cyclical sensitivity')
display('Pre vs. post')
%[round(gamma_post.b4b(:,1)-gamma_pre.b4b(:,1),3) round(sqrt(gamma_post.b4b(:,2).^2+gamma_pre.b4b(:,2).^2),3)]
%[round(gamma_post.b5b(:,1)-gamma_pre.b5b(:,1),3) round(sqrt(gamma_post.b5b(:,2).^2+gamma_pre.b5b(:,2).^2),3)]
[round(gamma_post.b45b(:,1)-gamma_pre.b45b(:,1),3) round(sqrt(gamma_post.b45b(:,2).^2+gamma_pre.b45b(:,2).^2),3)]
display('Pre vs. 1986 onwards')
%[round(gamma_1986.b4b(:,1)-gamma_pre.b4b(:,1),3) round(sqrt(gamma_1986.b4b(:,2).^2+gamma_pre.b4b(:,2).^2),3)]
%[round(gamma_1986.b5b(:,1)-gamma_pre.b5b(:,1),3) round(sqrt(gamma_1986.b5b(:,2).^2+gamma_pre.b5b(:,2).^2),3)]
[round(gamma_1986.b45b(:,1)-gamma_pre.b45b(:,1),3) round(sqrt(gamma_1986.b45b(:,2).^2+gamma_pre.b45b(:,2).^2),3)]

display('Comparison spatio-temporal estimates pre vs. post')
[round(par_post(1)-par_pre(1),3) round(sqrt(varcov_post(1,1)+varcov_pre(1,1)),3)]
[round(par_post(2)-par_pre(2),3) round(sqrt(varcov_post(2,2)+varcov_pre(2,2)),3)]
[round(par_post(27)-par_pre(27),3) round(sqrt(varcov_post(27,27)+varcov_pre(27,27)),3)]
display('Comparison spatio-temporal estimates pre vs. 1986')
[round(par_1986(1)-par_pre(1),3) round(sqrt(varcov_1986(1,1)+varcov_pre(1,1)),3)]
[round(par_1986(2)-par_pre(2),3) round(sqrt(varcov_1986(2,2)+varcov_pre(2,2)),3)]
[round(par_1986(27)-par_pre(27),3) round(sqrt(varcov_1986(27,27)+varcov_pre(27,27)),3)]
spatial_pre=par_pre(2)+par_pre(27);
spatial_post=par_post(2)+par_post(27);
spatial_1986=par_1986(2)+par_1986(27);
vcspatial_pre=varcov_pre(2,2)+varcov_pre(27,27)+2*varcov_pre(2,27);
vcspatial_post=varcov_post(2,2)+varcov_post(27,27)+2*varcov_post(2,27);
vcspatial_1986=varcov_1986(2,2)+varcov_1986(27,27)+2*varcov_1986(2,27);
display('Significance overall spatial estimate, i.e. when taken together')
[spatial_pre sqrt(vcspatial_pre) spatial_pre/sqrt(vcspatial_pre);...
    spatial_post sqrt(vcspatial_post) spatial_post/sqrt(vcspatial_post);...
    spatial_1986 sqrt(vcspatial_1986) spatial_1986/sqrt(vcspatial_1986)]
display('Comparison overall spatial estimate, i.e. when taken together pre vs. post')
[round(spatial_post-spatial_pre,3) round(sqrt(vcspatial_post+vcspatial_pre),3)]
display('Comparison overall spatial estimate, i.e. when taken together pre vs. 1986')
[round(spatial_1986-spatial_pre,3) round(sqrt(vcspatial_1986+vcspatial_pre),3)]

